@@ -7,11 +7,7 @@ import select
 import signal
 import sys
 import threading
-
-try:
-    import queue
-except ImportError:
-    import Queue as queue
+import queue
 
 from psshlib.askpass_server import PasswordServer
 from psshlib import psshutil
@@ -40,6 +36,7 @@ class Manager(object):
         self.askpass = opts.askpass
         self.outdir = opts.outdir
         self.errdir = opts.errdir
+        self.fileappend = opts.fileappend
         self.iomap = make_iomap()
 
         self.next_nodenum = 0
@@ -54,7 +51,7 @@ class Manager(object):
         """Processes tasks previously added with add_task."""
         try:
             if self.outdir or self.errdir:
-                writer = Writer(self.outdir, self.errdir)
+                writer = Writer(self.outdir, self.errdir, self.fileappend)
                 writer.start()
             else:
                 writer = None
@@ -93,9 +90,6 @@ class Manager(object):
         statuses = [task.exitstatus for task in self.done]
         return statuses
 
-    def clear_sigchld_handler(self):
-        signal.signal(signal.SIGCHLD, signal.SIG_DFL)
-
     def set_sigchld_handler(self):
         signal.signal(signal.SIGCHLD, self.handle_sigchld)
         # This should keep reads and writes from getting EINTR.
@@ -118,24 +112,13 @@ class Manager(object):
 
     def update_tasks(self, writer):
         """Reaps tasks and starts as many new ones as allowed."""
-        # Mask signals to work around a Python bug:
-        #   http://bugs.python.org/issue1068268
-        # Since sigprocmask isn't in the stdlib, clear the SIGCHLD handler.
-        # Since signals are masked, reap_tasks needs to be called once for
-        # each loop.
-        keep_running = True
-        while keep_running:
-            self.clear_sigchld_handler()
+        while True:
             self._start_tasks_once(writer)
-            self.set_sigchld_handler()
-            keep_running = self.reap_tasks()
+            if self.reap_tasks() == 0:
+                break
 
     def _start_tasks_once(self, writer):
-        """Starts tasks once.
-
-        Due to http://bugs.python.org/issue1068268, signals must be masked
-        when this method is called.
-        """
+        """Starts tasks once."""
         while 0 < len(self.tasks) and len(self.running) < self.limit:
             task = self.tasks.pop(0)
             self.running.append(task)
@@ -332,14 +315,19 @@ class Writer(threading.Thread):
     EOF = object()
     ABORT = object()
 
-    def __init__(self, outdir, errdir):
+    def __init__(self, outdir, errdir, fileappend):
         threading.Thread.__init__(self)
         # A daemon thread automatically dies if the program is terminated.
-        self.setDaemon(True)
+        self.daemon = True
         self.queue = queue.Queue()
         self.outdir = outdir
         self.errdir = errdir
 
+        if fileappend:
+            self.filewritemode = 'ab'
+        else:
+            self.filewritemode = 'wb'
+
         self.host_counts = {}
         self.files = {}
 
@@ -359,10 +347,9 @@ class Writer(threading.Thread):
                 else:
                     if dest is None:
                         dest = self.files[filename] = open(
-                            filename, 'wb', buffering=1)
+                            filename, self.filewritemode, buffering=0)
                         psshutil.set_cloexec(dest)
                     dest.write(data)
-                    dest.flush()
 
     def open_files(self, host):
         """Called from another thread to create files for stdout and stderr.
